/*
   Copyright The containerd Authors.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

syntax = "proto3";

package containerd.types.transfer;

option go_package = "github.com/containerd/containerd/api/types/transfer";

message Data {
  bytes data = 1;
}

message WindowUpdate {
  int32 update = 1;
}

// ReadStream carries data from the client to the server (import
// direction). The client sends data through the stream and the
// server reads it.
message ReadStream {
	string stream = 1;
	string media_type = 2;
}

// WriteStream carries data from the server to the client (export
// direction). The server writes data into the stream and the
// client receives it.
message WriteStream {
	string stream = 1;
	string media_type = 2;
}
