// Code generated by smithy-go-codegen DO NOT EDIT.

package cloudwatchlogs

import (
	"context"
	"github.com/aws/aws-sdk-go-v2/service/cloudwatchlogs/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Discovers available fields for a specific data source and type. The response
// includes any field modifications introduced through pipelines, such as new
// fields or changed field types.
func (c *Client) GetLogFields(ctx context.Context, params *GetLogFieldsInput, optFns ...func(*Options)) (*GetLogFieldsOutput, error) {
	if params == nil {
		params = &GetLogFieldsInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "GetLogFields", params, optFns, c.addOperationGetLogFieldsMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*GetLogFieldsOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type GetLogFieldsInput struct {

	// The name of the data source to retrieve log fields for.
	//
	// This member is required.
	DataSourceName *string

	// The type of the data source to retrieve log fields for.
	//
	// This member is required.
	DataSourceType *string

	noSmithyDocumentSerde
}

type GetLogFieldsOutput struct {

	// The list of log fields for the specified data source, including field names and
	// their data types.
	LogFields []types.LogFieldsListItem

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationGetLogFieldsMiddlewares(stack *middleware.Stack, options Options) (err error) {
	err = stack.Serialize.Add(&awsAwsjson11_serializeOpGetLogFields{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsAwsjson11_deserializeOpGetLogFields{}, middleware.After)
	if err != nil {
		return err
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addCredentialSource(stack, options); err != nil {
		return err
	}
	if err = addOpGetLogFieldsValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware(options.Region, "GetLogFields"), middleware.Before); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	if err = addInterceptors(stack, options); err != nil {
		return err
	}
	return nil
}
