/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.1
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

// Plan9ShareFlags is a bitfield controlling Plan9 share behavior.
type Plan9ShareFlags int32

const (
	// Plan9ShareFlagsNone indicates no flags are set.
	Plan9ShareFlagsNone Plan9ShareFlags = 0x00000000

	// Plan9ShareFlagsReadOnly makes the share read-only.
	Plan9ShareFlagsReadOnly Plan9ShareFlags = 0x00000001

	// Plan9ShareFlagsLinuxMetadata enables writing Linux metadata.
	Plan9ShareFlagsLinuxMetadata Plan9ShareFlags = 0x00000004

	// Plan9ShareFlagsCaseSensitive creates directories in case-sensitive mode.
	Plan9ShareFlagsCaseSensitive Plan9ShareFlags = 0x00000008

	// Plan9ShareFlagsUseShareRootIdentity uses the identity of the share root
	// when opening.
	Plan9ShareFlagsUseShareRootIdentity Plan9ShareFlags = 0x00000010

	// Plan9ShareFlagsRestrictFileAccess enables single-file mapping mode.
	Plan9ShareFlagsRestrictFileAccess Plan9ShareFlags = 0x00000080

	// Plan9ShareFlagsUnlimitedConnections allows multiple connections to this
	// share.
	Plan9ShareFlagsUnlimitedConnections Plan9ShareFlags = 0x00000100
)
