// Code generated by smithy-go-codegen DO NOT EDIT.

package cloudwatchlogs

import (
	"context"
	"github.com/aws/aws-sdk-go-v2/service/cloudwatchlogs/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Retrieves metadata about lookup tables in your account. You can optionally
// filter the results by table name prefix. Results are sorted by table name in
// ascending order.
func (c *Client) DescribeLookupTables(ctx context.Context, params *DescribeLookupTablesInput, optFns ...func(*Options)) (*DescribeLookupTablesOutput, error) {
	if params == nil {
		params = &DescribeLookupTablesInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "DescribeLookupTables", params, optFns, c.addOperationDescribeLookupTablesMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*DescribeLookupTablesOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type DescribeLookupTablesInput struct {

	// A prefix to filter lookup tables by name. Only tables whose names start with
	// this prefix are returned. If you don't specify a prefix, all tables in the
	// account and Region are returned.
	LookupTableNamePrefix *string

	// The maximum number of lookup tables to return in the response. The default
	// value is 50 and the maximum value is 100.
	MaxResults int32

	// The token for the next set of items to return. (You received this token from a
	// previous call.)
	NextToken *string

	noSmithyDocumentSerde
}

type DescribeLookupTablesOutput struct {

	// An array of structures, where each structure contains metadata about one lookup
	// table.
	LookupTables []types.LookupTable

	// The token to use when requesting the next set of items.
	NextToken *string

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationDescribeLookupTablesMiddlewares(stack *middleware.Stack, options Options) (err error) {
	err = stack.Serialize.Add(&awsAwsjson11_serializeOpDescribeLookupTables{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsAwsjson11_deserializeOpDescribeLookupTables{}, middleware.After)
	if err != nil {
		return err
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addCredentialSource(stack, options); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware(options.Region, "DescribeLookupTables"), middleware.Before); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	if err = addInterceptors(stack, options); err != nil {
		return err
	}
	return nil
}
