/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.4
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

type Processor struct {
	Count int32 `json:"Count,omitempty"`

	Maximum int32 `json:"Maximum,omitempty"`

	Weight int32 `json:"Weight,omitempty"`
}
